﻿using System;
using System.Windows.Media;

namespace DynamicColors.Utilities
{
    public class HslColor
    {
        public HslColor(Color color)
        { FromRgb(color); }

        public double Hue
        { get; set; }

        public double Saturation
        { get; set; }

        public double Luminance
        { get; set; }

        public Color Color
        {
            get { return ToRgb(); }
        }

        private void FromRgb(Color color)
        {
            // r, g, b -> [0, 1]
            var r = color.R / 255.0;
            var g = color.G / 255.0;
            var b = color.B / 255.0;

            var min = Math.Min(r, Math.Min(g, b));
            var max = Math.Max(r, Math.Max(g, b));

            // Hue -> [0, 360]
            if (max == min)
                Hue = 0.0;
            else if (r == max)
                Hue = (g - b) / (max - min);
            else if (g == max)
                Hue = 2.0 + (b - r) / (max - min);
            else// if (b == max)
                Hue = 4.0 + (r - g) / (max - min);

            Hue *= 60.0;

            if (Hue < 0.0)
                Hue += 360.0;

            // Luminance -> [0,1]
            Luminance = (max + min) / 2.0;

            // Saturation -> [0, 1]
            if (max == min)
                Saturation = 0.0;
            else if (Luminance < 0.5)
                Saturation = (max - min) / (max + min);
            else// if (Luminance >= 0.5)
                Saturation = (max - min) / (2.0 - (max + min));
        }

        private Color ToRgb()
        {
            double r, g, b;

            if (Saturation == 0.0)
            {
                r = g = b = Luminance;
            }
            else
            {
                double temp2;

                if (Luminance < 0.5)
                    temp2 = Luminance * (1.0 + Saturation);
                else
                    temp2 = Luminance + Saturation - Luminance * Saturation;

                var temp1 = 2.0 * Luminance - temp2;

                var hue = Hue / 360.0;

                // Red
                var temp_r = hue + 1.0 / 3.0;

                if (temp_r < 0.0)
                    temp_r += 1.0;
                else if (temp_r > 1.0)
                    temp_r -= 1.0;

                if ((6.0 * temp_r) < 1.0)
                    r = temp1 + (temp2 - temp1) * 6.0 * temp_r;
                else if ((2.0 * temp_r) < 1.0)
                    r = temp2;
                else if ((3.0 * temp_r) < 2.0)
                    r = temp1 + (temp2 - temp1) * ((2.0 / 3.0) - temp_r) * 6.0;
                else
                    r = temp1;

                // Green
                var temp_g = hue;

                if (temp_g < 0.0)
                    temp_g += 1.0;
                else if (temp_g > 1.0)
                    temp_g -= 1.0;

                if ((6.0 * temp_g) < 1.0)
                    g = temp1 + (temp2 - temp1) * 6.0 * temp_g;
                else if ((2.0 * temp_g) < 1.0)
                    g = temp2;
                else if ((3.0 * temp_g) < 2.0)
                    g = temp1 + (temp2 - temp1) * ((2.0 / 3.0) - temp_g) * 6.0;
                else
                    g = temp1;

                // Blue
                var temp_b = hue - 1.0 / 3.0;

                if (temp_b < 0.0)
                    temp_b += 1.0;
                else if (temp_b > 1.0)
                    temp_b -= 1.0;

                if ((6.0 * temp_b) < 1.0)
                    b = temp1 + (temp2 - temp1) * 6.0 * temp_b;
                else if ((2.0 * temp_b) < 1.0)
                    b = temp2;
                else if ((3.0 * temp_b) < 2.0)
                    b = temp1 + (temp2 - temp1) * ((2.0 / 3.0) - temp_b) * 6.0;
                else
                    b = temp1;
            }

            return Color.FromRgb(
                Convert.ToByte(r * 255),
                Convert.ToByte(g * 255),
                Convert.ToByte(b * 255)
            );
        }
    }
}
